import os
import random
import datetime
import uuid
import glob
import time
import json
import base64

import tator

def random_localization(project, box_type, media_ids):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        'test_float_array': [random.uniform(-1.0, 1.0) for _ in range(3)],
    }
    out = {
        'x': x,
        'y': y,
        'width': w,
        'height': h,
        'project': project,
        'type': box_type,
        'media_id': random.choice(media_ids),
        'frame': 0,
        'attributes': attributes
    }

    return {**out}

def random_media(api, project, paths, image_type):
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        'test_float_array': [random.uniform(-1.0, 1.0) for _ in range(3)],
    }
    section = random.choice(['Search Section A', 'Search Section B'])
    path = random.choice(paths)
    for _, response in tator.util.upload_media(
        api, image_type, path, attributes=attributes, section=section
    ):
        pass
    return response.id, attributes, section

def random_search(sections=None):
    """ Runs a random query and compares results with ES enabled and disabled.
    """
    bool_value = random.choice([True, False])
    int_lower = random.randint(-1000, 0)
    int_upper = random.randint(0, 1000)
    float_lower = random.uniform(-1000.0, 0.0)
    float_upper = random.uniform(0.0, 1000.0)
    enum_value = random.choice(['a', 'b', 'c'])
    search = {"method": 'AND', "operations": [
        {'attribute': 'test_bool', 'operation': 'eq', 'value': bool_value},
        {'attribute': 'test_int', 'operation': 'lt', 'value': int_upper},
        {'attribute': 'test_int', 'operation': 'gt', 'value': int_lower},
        {'attribute': 'test_float', 'operation': 'gt', 'value': float_lower},
        {'attribute': 'test_float', 'operation': 'lt', 'value': float_upper},
        {'attribute': 'test_enum', 'operation': 'eq', 'value': enum_value},
    ]
    }
    values = {'bool_value': bool_value,
              'int_lower': int_lower,
              'int_upper': int_upper,
              'float_lower': float_lower,
              'float_upper': float_upper,
              'enum_value': enum_value,
              }
    if sections:
        section = random.choice(sections)
        search['operations'].append({'attribute':'tator_user_sections', 'operation': 'eq', 'value': section.tator_user_sections})
        values['section'] = section
    return search, values

def check_box_result(results, values, media_values, box_specs, medias):
    """ Checks search results against values cached locally.
    """
    expected_media = [media_id for media_id, attributes, section in medias
                      if (attributes['test_bool'] == media_values['bool_value'])
                      and (attributes['test_int'] > media_values['int_lower'])
                      and (attributes['test_int'] < media_values['int_upper'])
                      and (attributes['test_float'] > media_values['float_lower'])
                      and (attributes['test_float'] < media_values['float_upper'])
                      and (attributes['test_enum'] == media_values['enum_value'])
                      and (section == media_values['section'].name)]
    expected_boxes = [box for box in box_specs
                      if (box['media_id'] in expected_media)
                      and (box['attributes']['test_bool'] == values['bool_value'])
                      and (box['attributes']['test_int'] > values['int_lower'])
                      and (box['attributes']['test_int'] < values['int_upper'])
                      and (box['attributes']['test_float'] > values['float_lower'])
                      and (box['attributes']['test_float'] < values['float_upper'])
                      and (box['attributes']['test_enum'] == values['enum_value'])]
    box_ids = [box['id'] for box in expected_boxes]
    assert(len(expected_boxes) == len(results))
    for result in results:
        assert(result.id in box_ids)

def check_media_result(results, values, annotation_values, box_specs, medias):
    """ Checks search results against values cached locally.
    """
    expected_boxes = [box for box in box_specs
                      if (box['attributes']['test_bool'] == annotation_values['bool_value'])
                      and (box['attributes']['test_int'] > annotation_values['int_lower'])
                      and (box['attributes']['test_int'] < annotation_values['int_upper'])
                      and (box['attributes']['test_float'] > annotation_values['float_lower'])
                      and (box['attributes']['test_float'] < annotation_values['float_upper'])
                      and (box['attributes']['test_enum'] == annotation_values['enum_value'])]
    parent_media = [box['media_id'] for box in expected_boxes]
    expected_media = [media_id for media_id, attributes, section in medias
                      if (media_id in parent_media)
                      and (attributes['test_bool'] == values['bool_value'])
                      and (attributes['test_int'] > values['int_lower'])
                      and (attributes['test_int'] < values['int_upper'])
                      and (attributes['test_float'] > values['float_lower'])
                      and (attributes['test_float'] < values['float_upper'])
                      and (attributes['test_enum'] == values['enum_value'])
                      and (section == values['section'].name)]
    assert(len(expected_media) == len(results))
    for result in results:
        assert(result.id in expected_media)

def test_search(host, token, project, image_type, image_set, box_type):
    api = tator.get_api(host, token)
    paths = os.listdir(image_set)
    paths = glob.glob(os.path.join(image_set, '**/*.jpg'), recursive=True)
    # Create some random media.
    print("Uploading 20 images...")
    medias = [random_media(api, project, paths, image_type) for _ in range(20)]
    media_ids = [media[0] for media in medias]
    # Create some random boxes.
    print("Creating 500 boxes...")
    box_specs = [random_localization(project, box_type, media_ids) for _ in range(500)]
    response = api.create_localization_list(project, box_specs)
    for idx, box_id in enumerate(response.id):
        box_specs[idx]['id'] = box_id
    # Retrieve sections.
    sections = [api.get_section_list(project, name='Search Section A')[0],
                api.get_section_list(project, name='Search Section B')[0]]
    # Sleep for a bit to make sure localizations are indexed.
    time.sleep(10)
    # Test search on localizations.
    print("Performing 100 random searches on localizations...")
    for _ in range(100):
        print(f"Iteration {_}")
        search, values = random_search()
        media_search, media_values = random_search(sections)
        blob=base64.b64encode(json.dumps(search).encode())
        media_blob=base64.b64encode(json.dumps(media_search).encode())
        response = api.get_localization_list(project, encoded_search=blob, encoded_related_search=media_blob)
        check_box_result(response, values, media_values, box_specs, medias)
        # Test search with section parameter
        media_search, media_values = random_search()
        media_blob=base64.b64encode(json.dumps(media_search).encode())
        section = random.choice(sections)
        response = api.get_localization_list(project, encoded_search=blob, encoded_related_search=media_blob,
                                             section=section.id)
        media_values['section'] = section
        check_box_result(response, values, media_values, box_specs, medias)
    # Test search on media.
    print("Performing 100 random searches on media...")
    for _ in range(100):
        search, values = random_search(sections)
        annotation_search, annotation_values = random_search()
        blob=base64.b64encode(json.dumps(search).encode('ascii'))
        annotation_blob=base64.b64encode(json.dumps(annotation_search).encode('ascii'))
        response = api.get_media_list(project, encoded_search=blob, encoded_related_search=annotation_blob)
        check_media_result(response, values, annotation_values, box_specs, medias)
        # Test search with section parameter
        search, values = random_search()
        section = random.choice(sections)
        blob=base64.b64encode(json.dumps(search).encode('ascii'))
        response = api.get_media_list(project, encoded_search=blob, encoded_related_search=annotation_blob,
                                      section=section.id)
        values['section'] = section
        check_media_result(response, values, annotation_values, box_specs, medias)




# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class MediaType(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'archive_config': 'list[ArchiveConfig]',
        'attribute_types': 'list[AttributeType]',
        'default_box': 'int',
        'default_dot': 'int',
        'default_line': 'int',
        'default_volume': 'int',
        'description': 'str',
        'dtype': 'str',
        'effective_permission': 'int',
        'elemental_id': 'str',
        'file_format': 'str',
        'id': 'int',
        'name': 'str',
        'overlay_config': 'dict(str, object)',
        'project': 'int',
        'streaming_config': 'list[ResolutionConfig]',
        'visible': 'bool'
    }

    attribute_map = {
        'archive_config': 'archive_config',
        'attribute_types': 'attribute_types',
        'default_box': 'default_box',
        'default_dot': 'default_dot',
        'default_line': 'default_line',
        'default_volume': 'default_volume',
        'description': 'description',
        'dtype': 'dtype',
        'effective_permission': 'effective_permission',
        'elemental_id': 'elemental_id',
        'file_format': 'file_format',
        'id': 'id',
        'name': 'name',
        'overlay_config': 'overlay_config',
        'project': 'project',
        'streaming_config': 'streaming_config',
        'visible': 'visible'
    }

    def __init__(self, archive_config=None, attribute_types=None, default_box=None, default_dot=None, default_line=None, default_volume=None, description='', dtype=None, effective_permission=None, elemental_id=None, file_format=None, id=None, name=None, overlay_config=None, project=None, streaming_config=None, visible=None, local_vars_configuration=None):  # noqa: E501
        """MediaType - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._archive_config = None
        self._attribute_types = None
        self._default_box = None
        self._default_dot = None
        self._default_line = None
        self._default_volume = None
        self._description = None
        self._dtype = None
        self._effective_permission = None
        self._elemental_id = None
        self._file_format = None
        self._id = None
        self._name = None
        self._overlay_config = None
        self._project = None
        self._streaming_config = None
        self._visible = None
        self.discriminator = None

        if archive_config is not None:
            self.archive_config = archive_config
        if attribute_types is not None:
            self.attribute_types = attribute_types
        if default_box is not None:
            self.default_box = default_box
        if default_dot is not None:
            self.default_dot = default_dot
        if default_line is not None:
            self.default_line = default_line
        if default_volume is not None:
            self.default_volume = default_volume
        if description is not None:
            self.description = description
        if dtype is not None:
            self.dtype = dtype
        if effective_permission is not None:
            self.effective_permission = effective_permission
        self.elemental_id = elemental_id
        if file_format is not None:
            self.file_format = file_format
        if id is not None:
            self.id = id
        if name is not None:
            self.name = name
        if overlay_config is not None:
            self.overlay_config = overlay_config
        if project is not None:
            self.project = project
        if streaming_config is not None:
            self.streaming_config = streaming_config
        if visible is not None:
            self.visible = visible

    @property
    def archive_config(self):
        """
        Archive config definitions. If null, the raw file will be uploaded to Tator.

        :return: The archive_config of this MediaType. 
        :rtype: list[ArchiveConfig]
        """
        return self._archive_config

    @archive_config.setter
    def archive_config(self, archive_config):
        """
        Archive config definitions. If null, the raw file will be uploaded to Tator.

        :param archive_config: The archive_config of this MediaType.
        :type: list[ArchiveConfig]
        """

        self._archive_config = archive_config

    @property
    def attribute_types(self):
        """
        Attribute type definitions.

        :return: The attribute_types of this MediaType. 
        :rtype: list[AttributeType]
        """
        return self._attribute_types

    @attribute_types.setter
    def attribute_types(self, attribute_types):
        """
        Attribute type definitions.

        :param attribute_types: The attribute_types of this MediaType.
        :type: list[AttributeType]
        """

        self._attribute_types = attribute_types

    @property
    def default_box(self):
        """
        Unique integer identifying default box type for drawing.

        :return: The default_box of this MediaType. 
        :rtype: int
        """
        return self._default_box

    @default_box.setter
    def default_box(self, default_box):
        """
        Unique integer identifying default box type for drawing.

        :param default_box: The default_box of this MediaType.
        :type: int
        """

        self._default_box = default_box

    @property
    def default_dot(self):
        """
        Unique integer identifying default dot type for drawing.

        :return: The default_dot of this MediaType. 
        :rtype: int
        """
        return self._default_dot

    @default_dot.setter
    def default_dot(self, default_dot):
        """
        Unique integer identifying default dot type for drawing.

        :param default_dot: The default_dot of this MediaType.
        :type: int
        """

        self._default_dot = default_dot

    @property
    def default_line(self):
        """
        Unique integer identifying default line type for drawing.

        :return: The default_line of this MediaType. 
        :rtype: int
        """
        return self._default_line

    @default_line.setter
    def default_line(self, default_line):
        """
        Unique integer identifying default line type for drawing.

        :param default_line: The default_line of this MediaType.
        :type: int
        """

        self._default_line = default_line

    @property
    def default_volume(self):
        """
        Default audio volume for this media type.

        :return: The default_volume of this MediaType. 
        :rtype: int
        """
        return self._default_volume

    @default_volume.setter
    def default_volume(self, default_volume):
        """
        Default audio volume for this media type.

        :param default_volume: The default_volume of this MediaType.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                default_volume is not None and default_volume > 100):  # noqa: E501
            raise ValueError("Invalid value for `default_volume`, must be a value less than or equal to `100`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                default_volume is not None and default_volume < 0):  # noqa: E501
            raise ValueError("Invalid value for `default_volume`, must be a value greater than or equal to `0`")  # noqa: E501

        self._default_volume = default_volume

    @property
    def description(self):
        """
        Description of the media type.

        :return: The description of this MediaType. 
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """
        Description of the media type.

        :param description: The description of this MediaType.
        :type: str
        """

        self._description = description

    @property
    def dtype(self):
        """
        Type of the media.

        :return: The dtype of this MediaType. 
        :rtype: str
        """
        return self._dtype

    @dtype.setter
    def dtype(self, dtype):
        """
        Type of the media.

        :param dtype: The dtype of this MediaType.
        :type: str
        """
        allowed_values = ["image", "video", "multi", "live"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and dtype not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `dtype` ({0}), must be one of {1}"  # noqa: E501
                .format(dtype, allowed_values)
            )

        self._dtype = dtype

    @property
    def effective_permission(self):
        """
        Effective permission mask for this entity.

        :return: The effective_permission of this MediaType. 
        :rtype: int
        """
        return self._effective_permission

    @effective_permission.setter
    def effective_permission(self, effective_permission):
        """
        Effective permission mask for this entity.

        :param effective_permission: The effective_permission of this MediaType.
        :type: int
        """

        self._effective_permission = effective_permission

    @property
    def elemental_id(self):
        """
        The elemental ID of the object.

        :return: The elemental_id of this MediaType. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        The elemental ID of the object.

        :param elemental_id: The elemental_id of this MediaType.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def file_format(self):
        """
        File extension. If omitted, any recognized file extension for the given dtype is accepted for upload. Do not include a dot prefix.

        :return: The file_format of this MediaType. 
        :rtype: str
        """
        return self._file_format

    @file_format.setter
    def file_format(self, file_format):
        """
        File extension. If omitted, any recognized file extension for the given dtype is accepted for upload. Do not include a dot prefix.

        :param file_format: The file_format of this MediaType.
        :type: str
        """
        if (self.local_vars_configuration.client_side_validation and
                file_format is not None and len(file_format) > 4):
            raise ValueError("Invalid value for `file_format`, length must be less than or equal to `4`")  # noqa: E501

        self._file_format = file_format

    @property
    def id(self):
        """
        Unique integer identifying a media type.

        :return: The id of this MediaType. 
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """
        Unique integer identifying a media type.

        :param id: The id of this MediaType.
        :type: int
        """

        self._id = id

    @property
    def name(self):
        """
        Name of the media type.

        :return: The name of this MediaType. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of the media type.

        :param name: The name of this MediaType.
        :type: str
        """

        self._name = name

    @property
    def overlay_config(self):
        """
        Overlay configuration

        :return: The overlay_config of this MediaType. 
        :rtype: dict(str, object)
        """
        return self._overlay_config

    @overlay_config.setter
    def overlay_config(self, overlay_config):
        """
        Overlay configuration

        :param overlay_config: The overlay_config of this MediaType.
        :type: dict(str, object)
        """

        self._overlay_config = overlay_config

    @property
    def project(self):
        """
        Unique integer identifying project for this media type.

        :return: The project of this MediaType. 
        :rtype: int
        """
        return self._project

    @project.setter
    def project(self, project):
        """
        Unique integer identifying project for this media type.

        :param project: The project of this MediaType.
        :type: int
        """

        self._project = project

    @property
    def streaming_config(self):
        """
        Streaming config definition. If null, the default will be used.

        :return: The streaming_config of this MediaType. 
        :rtype: list[ResolutionConfig]
        """
        return self._streaming_config

    @streaming_config.setter
    def streaming_config(self, streaming_config):
        """
        Streaming config definition. If null, the default will be used.

        :param streaming_config: The streaming_config of this MediaType.
        :type: list[ResolutionConfig]
        """

        self._streaming_config = streaming_config

    @property
    def visible(self):
        """
        Visible configuration

        :return: The visible of this MediaType. 
        :rtype: bool
        """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """
        Visible configuration

        :param visible: The visible of this MediaType.
        :type: bool
        """

        self._visible = visible

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MediaType):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MediaType):
            return True

        return self.to_dict() != other.to_dict()



import os

from ._upload_file import _upload_file


def upload_attachment(api, media, path, name=None, chunk_size=1 * 1024 * 1024):
    """Upload a file as an attachment to a media object.

    Example:

    .. code-block:: python

        api = tator.get_api(host, token)
        for progress, response in tator.util.upload_attachment(api, media_id, path):
            print(f"Upload progress: {progress}%")
        print(response.message)

    :param api: :class:`tator.TatorApi` object.
    :param media: Unique integer identifying a media.
    :param path: Path to the file.
    :param name: [Optional] Name of file in database. Defaults to basename of path.
    :param chunk_size: [Optional] Chunk size in bytes. Default is 10MB.
    :returns: Generator that yields tuple containing progress (0-100) and a
        response. The response is `None` until the last yield, when the response
        is the response object from :meth:`tator.util.TatorApi.create_auxiliary_file`.
    """
    size = os.stat(path).st_size
    media_obj = api.get_media(media)

    if name is None:
        name = os.path.basename(path)

    for progress, upload_info in _upload_file(
        api, media_obj.project, path, media_id=media, filename=name, chunk_size=chunk_size
    ):
        yield (progress, None)

    file_def = {'size': os.stat(path).st_size,
                'path': upload_info.key,
                'name': name}

    response = api.create_auxiliary_file(media, role='attachment', auxiliary_file_definition=file_def)
    yield (100, response)



import tator

def test_clone_media_type_same_host(host, token, project, video_type, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator.util.clone_media_type(tator_api, video_type, clone_project)
    assert(isinstance(response, tator.models.CreateResponse))

def test_clone_media_type_different_host(host, token, project, video_type, clone_project):
    tator_api = tator.get_api(host, token)
    response = tator.util.clone_media_type(tator_api, video_type, clone_project, tator_api)
    assert(isinstance(response, tator.models.CreateResponse))




import random
import datetime
import uuid

import tator

def _create_localization(project, box_type, video_obj, float_array_val):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {
        'test_bool': random.choice([False, True]),
        'test_int': random.randint(-1000, 1000),
        'test_float': random.uniform(-1000.0, 1000.0),
        'test_enum': random.choice(['a', 'b', 'c']),
        'test_string': str(uuid.uuid1()),
        'test_datetime': datetime.datetime.now().isoformat(),
        'test_geopos': [random.uniform(-180.0, 180.0), random.uniform(-90.0, 90.0)],
        'test_float_array': float_array_val,
    }
    out = {
        'x': x,
        'y': y,
        'width': w,
        'height': h,
        'project': project,
        'type': box_type,
        'media_id': video_obj.id,
        'frame': random.randint(0, video_obj.num_frames - 1),
        'attributes': attributes
    }

    return {**out}

def test_float_array(host, token, project, box_type, video_temp):
    api = tator.get_api(host, token)
    video_obj = api.get_media(video_temp)

    # Create eight boxes in a line.
    spec = [
        _create_localization(project, box_type, video_obj, [-4.0, 0.0, 0.0]),
        _create_localization(project, box_type, video_obj, [-3.0, 0.0, 0.0]),
        _create_localization(project, box_type, video_obj, [-2.0, 0.0, 0.0]),
        _create_localization(project, box_type, video_obj, [-1.0, 0.0, 0.0]),
        _create_localization(project, box_type, video_obj, [0.0, 0.0, 0.0]),
        _create_localization(project, box_type, video_obj, [1.0, 0.0, 0.0]),
        _create_localization(project, box_type, video_obj, [2.0, 0.0, 0.0]),
        _create_localization(project, box_type, video_obj, [3.0, 0.0, 0.0]),
    ]
    response = api.create_localization_list(project, spec)
    assert(isinstance(response, tator.models.CreateListResponse))
    assert(len(spec) == len(response.id))
    ids = response.id

    # Test default vector search.
    search = {
        'name': 'test_float_array',
        'center': [2.1, 0.0, 0.0],
    }
    search = {'float_array': [search]}
    boxes = api.get_localization_list_by_id(project, type=box_type, media_id=[video_temp], localization_id_query=search)
    assert(len(boxes) == 8)

    # Test bounded search.
    search = {
        'name': 'test_float_array',
        'center': [2.1, 0.0, 0.0],
        'lower_bound': 1.0,
        'upper_bound': 3.0,
    }
    search = {'float_array': [search]}
    boxes = api.get_localization_list_by_id(project, type=box_type, media_id=[video_temp], localization_id_query=search, stop=1)
    assert(len(boxes) == 1)
    assert(boxes[0].attributes['test_float_array'][0] == 1.0)
    boxes = api.get_localization_list_by_id(project, type=box_type, media_id=[video_temp], localization_id_query=search, stop=1)

    # Test descending search.
    search = {
        'name': 'test_float_array',
        'center': [2.1, 0.0, 0.0],
        'lower_bound': 1.0,
        'upper_bound': 3.0,
        'order': 'desc',
    }
    search = {'float_array': [search]}
    boxes = api.get_localization_list_by_id(project, type=box_type,media_id=[video_temp],localization_id_query=search, stop=1)
    assert(len(boxes) == 1)
    assert(boxes[0].attributes['test_float_array'][0] == 0.0)

    # Update localizations.
    search = {
        'name': 'test_float_array',
        'center': [3.0, 0.0, 0.0],
        'upper_bound': 3.1,
    }
    search = {'float_array': [search]}
    update = {'attributes': {'test_float_array': [1000.0, 0.0, 0.0]}}
    response = api.update_localization_list(project, type=box_type,media_id=[video_temp],
                                            localization_bulk_update={**search, **update})
    assert(isinstance(response, tator.models.MessageResponse))

    # Delete localizations.
    search = {
        'name': 'test_float_array',
        'center': [1000.0, 0.0, 0.0],
        'upper_bound': 1.0,
    }
    search = {'float_array': [search]}
    response = api.delete_localization_list(project, type=box_type,media_id=[video_temp],localization_bulk_delete=search)
    assert(isinstance(response, tator.models.MessageResponse))

    # Check we deleted the right boxes.
    count = api.get_localization_count(project, type=box_type)
    assert(count == 4)

    # Delete remaining localizations.
    search = {
        'name': 'test_float_array',
        'center': [-2.0, 0.0, 0.0],
        'upper_bound': 2.1,
    }
    search = {'float_array': [search]}
    response = api.delete_localization_list(project, type=box_type,localization_bulk_delete=search,media_id=[video_temp])
    assert(isinstance(response, tator.models.MessageResponse))

    # Check localizations are gone.
    count = api.get_localization_count(project, type=box_type)
    assert(count == 0)



import tator
from tator.util._upload_file import _upload_file

def test_project_thumbnail(host, token, project, image_file):
    api = tator.get_api(host, token)
    for progress, upload_info in _upload_file(api, project, image_file):
        pass
    response = api.update_project(project, project_update={'thumb': upload_info.key})
    assert(isinstance(response, tator.models.MessageResponse))



import time
import subprocess

import tator
import uuid
import json


def _assert_subprocess(cmd, desired_result=True):
    out = subprocess.run(cmd, capture_output=True)

    if desired_result:
        assert out.returncode == 0, out.stderr
    else:
        assert out.returncode != 0, out.stdout


def _get_stream_info(path):
    cmd = [
        "ffprobe",
        "-v","error",
        "-show_entries", "stream",
        "-print_format", "json",
        "-select_streams", "v",
        path,
    ]
    p = subprocess.run(cmd, stdout=subprocess.PIPE)
    return json.loads(p.stdout.decode())['streams'][0]

def test_local_transcode(host, token, project, video_type, video_file):
    unique_name = f"{uuid.uuid4()}.mp4"
    cmd = [
        'python3', '-m', 'tator.transcode', video_file,
        '--host', host,
        '--token', token,
        '--project', str(project),
        '--type', str(video_type),
        '--section', 'Locally transcoded',
        '--name', unique_name
    ]
    _assert_subprocess(cmd)
    _assert_subprocess(cmd)
    api = tator.get_api(host, token)
    media_obj = api.get_media_list(project, name=unique_name, presigned=3600)[0]

    stream_info = _get_stream_info(media_obj.media_files.archival[0].path)
    assert stream_info['pix_fmt'] == 'yuv420p'
    stream_info = _get_stream_info(media_obj.media_files.streaming[0].path)
    assert stream_info['pix_fmt'] == 'yuv420p'


def test_local_transcode_inhibit(host, token, project, video_type, video_file):
    unique_name = f"{uuid.uuid4()}.mp4"
    cmd = [
        "python3",
        "-m",
        "tator.transcode",
        video_file,
        "--host",
        host,
        "--token",
        token,
        "--project",
        str(project),
        "--type",
        str(video_type),
        "--section",
        "Locally transcoded",
        "--name",
        unique_name,
        "--inhibit-upload",
    ]
    _assert_subprocess(cmd)
    api = tator.get_api(host, token)
    media_obj = api.get_media_list(project, name=unique_name, presigned=3600)[0]

    assert media_obj.media_files is None


def test_local_transcode_yuv444p(host, token, project, yuv444p_video_type, video_file):
    unique_name = f"{uuid.uuid4()}.mp4"
    cmd = [
        'python3', '-m', 'tator.transcode', video_file,
        '--host', host,
        '--token', token,
        '--project', str(project),
        '--type', str(yuv444p_video_type),
        '--section', 'Locally transcoded (yuv444p)',
        '--name', unique_name
    ]
    _assert_subprocess(cmd)
    api = tator.get_api(host, token)
    media_obj = api.get_media_list(project, name=unique_name, presigned=3600)[0]

    stream_info = _get_stream_info(media_obj.media_files.archival[0].path)
    assert stream_info['pix_fmt'] == 'yuv444p'
    stream_info = _get_stream_info(media_obj.media_files.streaming[0].path)
    assert stream_info['pix_fmt'] == 'yuv444p'

def test_bad_file(host, token, project, video_type, corrupted_video_file):
    cmd = [
        'python3', '-m', 'tator.transcode', corrupted_video_file,
        '--host', host,
        '--token', token,
        '--project', str(project),
        '--type', str(video_type),
        '--section', 'Bad transcodes',
    ]
    _assert_subprocess(cmd, False)
    time.sleep(2)
    # Make sure media file is gone.
    api = tator.get_api(host, token)
    sections = api.get_section_list(project)
    section_obj = [s for s in sections if s.name == 'Bad transcodes'][0]

    medias = api.get_media_list(project, attribute=[f'tator_user_sections::{section_obj.tator_user_sections}'])
    print(medias)
    assert(len(medias) == 0)

    medias = api.get_media_list(project, section=section_obj.id)
    print(medias)
    assert(len(medias) == 0)



import itertools

import tator

def full_state_graphic(api, state_id, force_scale=None):
    """ Retrieves all detection images for a state.

    Example:

    .. code-block:: python

        state_graphic = full_state_graphic(api, state_id)

    :param api: :class:`tator.TatorApi` object.
    :param state_id: Unique integer identifying a localization-associated state.
    :param force_scale: (Optional) wxh to force each tile prior to stitch.
    :returns: List containing all detection images in this state.
    """
    BATCH_SIZE = 30
    state = api.get_state(state_id)
    graphics = []
    for offset in range(0, len(state.localizations), BATCH_SIZE):
        kwargs = dict(mode='tile', length=BATCH_SIZE, offset=offset)
        if force_scale is not None:
            kwargs['force_scale'] = force_scale
        path = api.get_state_graphic(state_id, **kwargs)
        num_images = len(state.localizations[offset:offset+BATCH_SIZE])
        graphics.append(tator.util.get_images(path, state, num_images))
    return list(itertools.chain.from_iterable(graphics))



import os
from ..openapi import tator_openapi
from ..openapi.tator_openapi.models import CreateListResponse, CreateResponse


def get_api(host='https://cloud.tator.io', token=os.getenv('TATOR_TOKEN')):
    """ Retrieves a :class:`tator.api` instance using the given host and token.

    :param host: URL of host. Default is https://cloud.tator.io.
    :param token: API token.
    :returns: :class:`tator.api` object.
    """
    config = tator_openapi.Configuration()
    config.host = host
    if token:
        config.api_key['Authorization'] = token
        config.api_key_prefix['Authorization'] = 'Token'

    api = tator_openapi.TatorApi(tator_openapi.ApiClient(config))
    api._create_media_list_impl = api.create_media_list

    def create_media_list_wrapper(*args, **kwargs):
        response = api._create_media_list_impl(*args, **kwargs)
        if "id" in response and isinstance(response["id"], list):
            try:
                return CreateListResponse(**response)
            except Exception:
                return response
        try:
            return CreateResponse(**response)
        except Exception:
            return response

    api.create_media_list = create_media_list_wrapper
    def legacy_create_media(project, media_spec, **kwargs):
        return api.create_media_list(project, media_spec, **kwargs)

    api.create_media = legacy_create_media
    return api



import datetime
import random
import uuid

import tator


def random_localization(project, box_type, video_obj, post=False):
    x = random.uniform(0.0, 1.0)
    y = random.uniform(0.0, 1.0)
    w = random.uniform(0.0, 1.0 - x)
    h = random.uniform(0.0, 1.0 - y)
    attributes = {}
    out = {
        "x": x,
        "y": y,
        "width": w,
        "height": h,
        "project": project,
        "type": box_type,
        "media_id": video_obj.id,
        "frame": random.randint(0, video_obj.num_frames - 1),
        "attributes": attributes
    }

    return {**out}


def test_localization_type_delete(host, token, project, video_type, video):
    tator_api = tator.get_api(host, token)
    response = tator_api.create_localization_type(
        project,
        localization_type_spec={
            "name": "box_type",
            "description": "Test box type",
            "project": project,
            "media_types": [video_type],
            "dtype": "box",
            "attribute_types": [],
        },
    )
    box_type = response.id
    video_obj = tator_api.get_media(video)

    # Verify no localizations exist to start
    response = tator_api.get_localization_list(project, type=box_type)
    assert len(response) == 0

    # Test bulk create.
    num_localizations = random.randint(20, 100)
    boxes = [
        random_localization(project, box_type, video_obj, post=True)
        for _ in range(num_localizations)
    ]
    box_ids = []
    for response in tator.util.chunked_create(
        tator_api.create_localization_list, project, body=boxes
    ):
        box_ids += response.id
    assert len(box_ids) == len(boxes)
    print(f"Created {len(box_ids)} boxes!")

    # Verify list is the right length
    response = tator_api.get_localization_list(project, type=box_type)
    assert len(response) == num_localizations

    response = tator_api.delete_localization_type(box_type)

    assert str(box_type) in response.message, "Localization type id not found in delete response"
    assert (
        str(num_localizations) in response.message
    ), "Localization count not found in delete response"

    try:
        caught_it = False
        count = -1
        count = tator_api.get_localization_count(project, type=box_type)
    except:
        caught_it=True
    finally:
        assert caught_it or count == 0



from time import sleep

import tator

def test_media_type_delete(host, token, project, image_file):
    tator_api = tator.get_api(host, token)

    response = tator_api.create_media_type(project, media_type_spec={
        'name': 'image_type',
        'description': 'Test image type',
        'project': project,
        'dtype': 'image',
        'attribute_types': [],
    })
    image_type = response.id
    n_images = 10

    # Verify no media exist to start
    count = tator_api.get_media_count(project, type=image_type)
    assert count == 0

    for idx in range(n_images):
        for progress, response in tator.util.upload_media(tator_api, image_type, image_file):
            print(f"Upload image ({idx+1} of {n_images}) progress: {progress}%")
        print(response.message)

    n_retries = 0
    max_retries = 30
    while tator_api.get_media_count(project, type=image_type) != n_images:
        num_retries += 1
        assert num_retries < max_retries, "Failed uploading media"
        sleep(0.5)

    response = tator_api.delete_media_type(image_type)

    assert str(image_type) in response.message, "Media type id not found in delete response"
    assert str(n_images) in response.message, "Media count not found in delete response"

    try:
        caught_it = False
        count = -1
        count = tator_api.get_media_count(project, type=image_type)
    except:
        caught_it=True
    finally:
        assert caught_it or count == 0



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class ArchiveConfig(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'encode': 'EncodeConfig',
        'name': 'str',
        's3_storage': 'S3StorageConfig'
    }

    attribute_map = {
        'encode': 'encode',
        'name': 'name',
        's3_storage': 's3_storage'
    }

    def __init__(self, encode=None, name=None, s3_storage=None, local_vars_configuration=None):  # noqa: E501
        """ArchiveConfig - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._encode = None
        self._name = None
        self._s3_storage = None
        self.discriminator = None

        self.encode = encode
        if name is not None:
            self.name = name
        if s3_storage is not None:
            self.s3_storage = s3_storage

    @property
    def encode(self):
        """

        :return: The encode of this ArchiveConfig. 
        :rtype: EncodeConfig
        """
        return self._encode

    @encode.setter
    def encode(self, encode):
        """

        :param encode: The encode of this ArchiveConfig.
        :type: EncodeConfig
        """
        if self.local_vars_configuration.client_side_validation and encode is None:  # noqa: E501
            raise ValueError("Invalid value for `encode`, must not be `None`")  # noqa: E501

        self._encode = encode

    @property
    def name(self):
        """
        Name of this archive config, used for retrieval in case of multiple archive configs.

        :return: The name of this ArchiveConfig. 
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """
        Name of this archive config, used for retrieval in case of multiple archive configs.

        :param name: The name of this ArchiveConfig.
        :type: str
        """

        self._name = name

    @property
    def s3_storage(self):
        """

        :return: The s3_storage of this ArchiveConfig. 
        :rtype: S3StorageConfig
        """
        return self._s3_storage

    @s3_storage.setter
    def s3_storage(self, s3_storage):
        """

        :param s3_storage: The s3_storage of this ArchiveConfig.
        :type: S3StorageConfig
        """

        self._s3_storage = s3_storage

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ArchiveConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ArchiveConfig):
            return True

        return self.to_dict() != other.to_dict()



import requests
import math
import os
import logging
import requests
import time
import concurrent.futures
from urllib.parse import urlparse, parse_qs

import tator

logger = logging.getLogger(__name__)

MAX_RETRIES = 10 # Maximum retries on a given chunk.
GCP_CHUNK_MOD = 256 * 1024 # Chunk size must be a multiple of 256KB for Google Cloud Storage

def _upload_chunk(file_part, chunk_count, chunk_size, file_size, url, path, gcp_upload, default_etag_val, timeout):
    for attempt in range(MAX_RETRIES):
        try:
            kwargs = {"data": file_part, "timeout": timeout}
            if gcp_upload:
                first_byte = chunk_count * chunk_size
                last_byte = min(first_byte + chunk_size, file_size) - 1
                kwargs["headers"] = {
                    "Content-Length": str(last_byte - first_byte),
                    "Content-Range": f"bytes {first_byte}-{last_byte}/{file_size}",
                }
            response = requests.put(url, **kwargs)
            if response.status_code >= 400:
                raise RuntimeError(f"Upload of {path} chunk {chunk_count} failed ({response.text})!")
            etag_str = response.headers.get("ETag", default_etag_val)
            if etag_str == None:
                raise RuntimeError("No ETag in response!")
            return {
                "ETag": etag_str,
                "PartNumber": chunk_count + 1,
            }
        except Exception as e:
            logger.warning(f"Upload of {path} chunk {chunk_count} failed ({e})! Attempt "
                           f"{attempt + 1}/{MAX_RETRIES}")
            if attempt == MAX_RETRIES - 1:
                raise RuntimeError(f"Upload of {path} failed!")
            else:
                time.sleep(10 * attempt)
                logger.warning(f"Backing off for {10 * attempt} seconds...")


def _upload_file(
    api,
    project,
    path,
    media_id=None,
    filename=None,
    chunk_size=1024 * 1024 * 10,
    file_size=None,
    file_id=None,
    timeout=30,
    max_workers=10,
    gcp_compat=True,
    bucket_id=None,
):
    """Uploads a file.

    :param api: `tator.TatorApi` object.
    :param project: Unique integer identifying a project.
    :param path: Path to file on disk.
    :param media_id: [Optional] Media ID if this is an upload for existing media.
    :param filename: [Optional] Filename (only used if media ID is given).
    :param file_id: [Optional] File ID if this is an upload for existing File.
    :param chunk_size: [Optional] Upload chunk size in bytes.
    :param timeout: [Optional] Request timeout for uploads in seconds. Default is 30.
    :param max_workers: [Optional] Max workers for concurrent requests.
    :param gcp_compat: [Optional] If True, chunk size will be rounded up to the nearest multiple of 256KB for Google Cloud Storage compatibility.
    :param bucket_id: [Optional] Bucket ID for upload, will default to project bucket if not provided.
    """

    # Get number of chunks.
    if file_size is None or file_size <= 0:
        file_size = os.stat(path).st_size

    if math.ceil(file_size / chunk_size) > 10000:
        chunk_size = math.ceil(file_size / 9000)
        logger.warning(
            f"Number of chunks exceeds maximum of 10,000. Increasing chunk size to {chunk_size}."
        )

    if gcp_compat and chunk_size % GCP_CHUNK_MOD:
        logger.warning(
            f"Chunk size must be a multiple of 256KB for Google Cloud Storage compatibility"
        )
        chunk_size = math.ceil(chunk_size / GCP_CHUNK_MOD) * GCP_CHUNK_MOD

    if path.startswith('https://') or path.startswith('http://') and filename:
        filename = filename.split('?')[0]
    # Get upload info.
    num_chunks = math.ceil(file_size / chunk_size)
    upload_kwargs = {'num_parts': num_chunks}
    if media_id is not None:
        upload_kwargs['media_id'] = media_id
    if file_id is not None:
        upload_kwargs['file_id'] = file_id
    if filename is not None:
        upload_kwargs['filename'] = filename
    if bucket_id is not None:
        upload_kwargs["bucket_id"] = bucket_id
    upload_info = api.get_upload_info(project, **upload_kwargs)

    # Functor to wrap around file versus URL
    def get_data(path):
        if path.startswith('https://') or path.startswith('http://'):
            return requests.get(path, stream=True).raw
        else:
            return open(path, 'rb')

    # Determine if we need the microsoft header
    parsed_url = urlparse(upload_info.urls[0])
    query_params = parse_qs(parsed_url.query)
    ms_upload = 'sig' in query_params

    if num_chunks > 1:
        # Upload parts.
        parts = []
        yield (0, None)
        gcp_upload = upload_info.upload_id == upload_info.urls[0]
        if gcp_upload:
            max_workers = 1 # GCP does not handle parallel workers
        with get_data(path) as f:
            with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
                def complete_chunk(fs):
                    parts.append(fs.result())
                    progress = round((len(parts) / num_chunks) *100,1)
                    return (progress, None)
                futures = set()
                for chunk_count, url in enumerate(upload_info.urls):
                    file_part = f.read(chunk_size)
                    default_etag_val = str(chunk_count) if (gcp_upload or ms_upload) else None
                    future = executor.submit(_upload_chunk, file_part, chunk_count, chunk_size, file_size, url, path, gcp_upload, default_etag_val, timeout)
                    futures.add(future)
                    if len(futures) > max_workers:
                        done, futures = concurrent.futures.wait(futures, return_when=concurrent.futures.FIRST_COMPLETED)
                        for future in done:
                            yield complete_chunk(future)
                for future in concurrent.futures.as_completed(futures):
                    yield complete_chunk(future)

                # Complete the upload.
                completed = False
                count = 0
                while completed is False and count < MAX_RETRIES:
                    try:
                        count += 1
                        response = api.complete_upload(
                            project,
                            upload_completion_spec={
                                "key": upload_info.key,
                                "upload_id": upload_info.upload_id,
                                "parts": sorted(parts, key=lambda x: x["PartNumber"]),
                                "bucket_id": bucket_id,
                            },
                        )
                        if not isinstance(response, tator.models.MessageResponse):
                            raise RuntimeError(f"Upload completion failed!")
                        logger.info(response.message)
                        completed=True
                    except Exception as e:
                        logger.warning(e)
                        if count == MAX_RETRIES - 1:
                            raise RuntimeError(f"Upload of {path} failed!")
                        else:
                            time.sleep(10 * count)
                            logger.warning(f"Backing off for {10 * count} seconds...")
                        completed = False
    else:
        # Upload in single request.
        with get_data(path) as f:
            data = f.read()
            headers = {}
            if ms_upload:
                headers = {'x-ms-blob-type': 'BlockBlob'}
            for attempt in range(MAX_RETRIES):
                response = requests.put(upload_info.urls[0], data=data, timeout=timeout, headers=headers)
                if response.status_code < 400:
                    break
                else:
                    logger.warning(f"Upload of {path} failed ({response.text}) size={len(data)}! Attempt "
                                   f"{attempt + 1}/{MAX_RETRIES}")
                    if attempt == MAX_RETRIES - 1:
                        raise RuntimeError(f"Upload of {path} failed!")
                    else:
                        time.sleep(10 * attempt)
                        logger.warning(f"Backing off for {10 * attempt} seconds...")

    # Return the parts and upload info.
    yield (100, upload_info)



import tator
from tator.openapi.tator_openapi import CreateListResponse


def test_make_multi_stream(host, token, project, video, video_temp, multi_type):
    tator_api = tator.get_api(host, token)
    assert video != video_temp
    multi_ids = [video, video_temp]
    response = tator.util.make_multi_stream(
        tator_api, multi_type, [1, 2], 'Test multi', multi_ids, 'Multi Videos'
    )

    assert isinstance(response, CreateListResponse)
    multi_obj = tator_api.get_media(response.id[0])
    assert multi_obj.media_files.ids == multi_ids


def test_missing_gif(host, token, project, video, video_temp, multi_type):
    tator_api = tator.get_api(host, token)
    assert video != video_temp

    # Remove the gifs and mock get_media_list
    video_obj = tator_api.get_media(video)
    video_temp_obj = tator_api.get_media(video_temp)
    video_temp_obj.media_files.thumbnail_gif = []
    video_obj.media_files.thumbnail_gif = []
    def get_media_list(*_, **__):
        return [video_temp_obj, video_obj]
    original_get_media_list = tator_api.get_media_list
    tator_api.get_media_list = get_media_list

    multi_ids = [video_temp, video]
    response = tator.util.make_multi_stream(
        tator_api, multi_type, [1, 2], 'Test multi', multi_ids, 'Multi Videos'
    )

    # Replace get_media_list
    tator_api.get_media_list = original_get_media_list

    assert isinstance(response, CreateListResponse)
    multi_obj = tator_api.get_media(response.id[0])
    assert multi_obj.media_files.ids == multi_ids



#!/usr/bin/env python3

""" Uses ENV variables to dispatch the frame extractor 

Generic TATOR workflow env variables this module expects:

TATOR_API_SERVICE : URL to host, e.g. https://cloud.tator.io
TATOR_AUTH_TOKEN : Token to use for authentication
TATOR_PROJECT_ID : numerical id of the project this algorithm is being run
TATOR_MEDIA_IDS : Comma-seperated list of media ids to process

:Workflow variables:
`python3 -m tator.extractor --help` for more information on the following
options:
EXTRACT_MODE : Maps to the `--mode` argument
METADATA_TYPE_ID : Maps to the `--metadata-type-id` argument
OUTPUT_SECTION : Maps to the `--tator-output-section` argument
OUTPUT_TYPE_ID : Maps to the `--output-type-id` argument  

"""
    
import os
import pandas as pd
import subprocess
import json
import sys
import shutil

if __name__=="__main__":
    media_ids = os.getenv('TATOR_MEDIA_IDS').split(',')
    cmd = ["python3",
           "-m", "tator.extractor",
           '--host', os.getenv("TATOR_API_SERVICE").replace('/rest',''),
           "--token",
           os.getenv("TATOR_AUTH_TOKEN"),
           "--mode", os.getenv("EXTRACT_MODE"),
           "--metadata-type-id", os.getenv("METADATA_TYPE_ID"),
           "--project", os.getenv("TATOR_PROJECT_ID"),
           "--output-tator-section", os.getenv("OUTPUT_SECTION"),
           "--output-type-id", os.getenv("OUTPUT_TYPE_ID"),
           # Assume PVC is located at work
           "--work-dir", "/work",
           *media_ids]

    p=subprocess.Popen(cmd)
    p.wait()
    sys.exit(p.returncode)
    



import logging
from math import floor
from typing import Callable
import time

logger = logging.getLogger(__name__)


def chunked_create(func: Callable, project: int, chunk_size: int = 500, **kwargs):
    """Breaks a create_*_list operation into chunks.

    Example:

    .. code-block:: python

        created_ids = [
            new_id
            for response in tator.util.chunked_create(
                api.create_localization_list, chunk_size=100, body=my_long_list
            )
            for new_id in response.id
        ]

    :param func: Function to call on each chunk.
    :param project: Unique integer identifying a project.
    :param chunk_size: [Optional] Sets the size of the chunks, defaults to 500.
    :param kwargs: Exactly one keyword argument should be set, which contains the list of specs to
                   create and is passed to `func`
    :returns: Generator that yields a response.
    """
    if len(kwargs) != 1:
        raise RuntimeError(f"Expects exactly one keyword argument")

    key = list(kwargs.keys())[0]
    spec = kwargs[key]
    n_specs = len(spec)
    while chunk_size > 0 and spec:
        n_created = 0
        try:
            for idx in range(0, len(spec), chunk_size):
                try:
                    response = func(project, **{key: spec[idx : idx + chunk_size]})
                    n_created += len(response.id)
                    yield response
                except Exception:
                    # Back-off once, wait 5 seconds, and try again. This helps work around
                    # if we trip a DDOS firewall in a bulk import
                    time.sleep(5)
                    response = func(project, **{key: spec[idx : idx + chunk_size]})
                    n_created += len(response.id)
                    yield response
        except Exception:
            chunk_size = floor(chunk_size / 2)
            logger.warning("Caught exception, halving chunk size to %d", chunk_size, exc_info=True)
        spec = spec[n_created:]
    if spec:
        raise RuntimeError(f"Was not able to create {len(spec)} of {n_specs} objects")



#!/usr/bin/env python

import argparse
import subprocess
import os
import json
import logging

from PIL import Image

from ..util import get_api
from ..util._upload_file import _upload_file

from .transcode import get_length_info
from ..openapi.tator_openapi.models import MessageResponse
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

def parse_args():
    parser = argparse.ArgumentParser(description='Makes thumbnails for a video.')
    parser.add_argument('--host', type=str, default='https://cloud.tator.io', help='Host URL.')
    parser.add_argument('--token', type=str, help='REST API token.')
    parser.add_argument('--media', type=int, help='Unique integer identifying a media.')
    parser.add_argument('input', type=str, help='Path to input file.')
    parser.add_argument("-o", "--output", type=str, help='Path to output thumbnail.')
    parser.add_argument("-g", "--gif", type=str, help='Path to output thumbnail gif.')
    return parser.parse_args()

def get_metadata(path):
    cmd = [
        "ffprobe",
        "-v","error",
        "-show_entries", "stream:format=duration",
        "-print_format", "json",
        "-select_streams", "v",
        "{}".format(path)
    ]
    output = subprocess.run(cmd, stdout=subprocess.PIPE, check=True).stdout

    logger.info("Got info = {}".format(output))
    video_info = json.loads(output)
    stream_idx=0
    for idx, stream in enumerate(video_info["streams"]):
        if stream["codec_type"] == "video":
            stream_idx=idx
            break
    stream = video_info["streams"][stream_idx]
    stream = {**video_info.get("format", {}), **stream}

    fps, num_frames = get_length_info(stream)
    # Fill in object information based on probe
    codec = stream["codec_name"]
    width = stream["width"]
    height = stream["height"]

    return (codec, fps, num_frames, width, height)


def make_thumbnail_image(host, token, media_id, video_path, thumb_path, inhibit_upload=False):
    # Check for the existence of thumbnails
    api = get_api(host, token)
    media_obj = api.get_media(media_id)

    if (media_obj.media_files and media_obj.media_files.thumbnail):
        logger.info(f"Thumbnail upload skipped for media '{media_id}', they already exist")
        return

    # Create thumbnail.
    cmd = ["ffmpeg", "-y", 
            "-loglevel", "error",
            "-progress", "-",
            "-stats_period", "10",
            "-i", video_path,
            "-vf", "scale=256:-1",
            "-vframes", "1", thumb_path]
    subprocess.run(cmd, check=True)

    if inhibit_upload:
        return

    for progress, thumbnail_info in _upload_file(
        api,
        media_obj.project,
        thumb_path,
        media_id=media_id,
        filename=os.path.basename(thumb_path),
    ):
        pass

    thumb_image = Image.open(thumb_path)
    thumb_def = {
        "path": thumbnail_info.key,
        "size": os.stat(thumb_path).st_size,
        "resolution": [thumb_image.height, thumb_image.width],
        "mime": f"image/{thumb_image.format.lower()}",
    }

    response = api.create_image_file(media_id, role="thumbnail", image_definition=thumb_def)
    assert isinstance(response, MessageResponse)


def make_thumbnail_gif(
    host, token, media_id, video_path, thumb_gif_path, only_keyframes=False, inhibit_upload=False
):
    """ Makes thumbnails and gets metadata for original file.
    """
    # Check for the existence of thumbnails
    api = get_api(host, token)
    media_obj = api.get_media(media_id)

    needs_thumb_gif = not (media_obj.media_files and media_obj.media_files.thumbnail_gif)

    if not needs_thumb_gif:
        logger.info(f"Thumbnail upload skipped for media '{media_id}', they already exist")
        return

    # Get metadata for original file.
    codec, fps, num_frames, width, height = get_metadata(video_path)

    # Create gif thumbnail in a single pass
    # This logic makes a max(video_length,60) second summary video than speeds it up 4 times and saves as a gif
    video_duration = num_frames/fps

    # Max thumbnail duration is 60 seconds
    thumb_duration = min(60, video_duration)

    # We either select every Nth second based on how much longer than 60 seconds we are
    frame_select = max(fps, (video_duration/thumb_duration)*fps)

    # We play each 1 second sample at 4x
    speed_up = 4*max(1,round(video_duration/thumb_duration))
    cmd = ["ffmpeg", "-y",
            "-loglevel", "error",
            "-progress", "-",
            "-stats_period", "10",
            "-i",
            video_path,
            "-vf",
            f"select='not(mod(n\,{round(frame_select)}))',scale=256:-1:flags=lanczos,setpts=PTS/{speed_up},split[s0][s1];[s0]palettegen[p];[s1][p]paletteuse",
            thumb_gif_path,
    ]
    logger.info(f"cmd={cmd}")
    subprocess.run(cmd, check=True)

    if inhibit_upload:
        return

    # Upload thumbnail gif.
    for progress, thumbnail_gif_info in _upload_file(
        api,
        media_obj.project,
        thumb_gif_path,
        media_id=media_id,
        filename=os.path.basename(thumb_gif_path),
    ):
        pass

    thumb_gif_image = Image.open(thumb_gif_path)
    thumb_gif_def = {
        "path": thumbnail_gif_info.key,
        "size": os.stat(thumb_gif_path).st_size,
        "resolution": [thumb_gif_image.height, thumb_gif_image.width],
        "mime": f"image/{thumb_gif_image.format.lower()}",
    }

    response = api.create_image_file(
        media_id, role="thumbnail_gif", image_definition=thumb_gif_def
    )
    assert isinstance(response, MessageResponse)

    logger.info(f'Thumbnail upload done! {response.message}')


def make_thumbnails(host, token, media_id, video_path, thumb_path, thumb_gif_path, only_keyframes=False):
    make_thumbnail_image(host, token, media_id, video_path, thumb_path)
    make_thumbnail_gif(host, token, media_id, video_path, thumb_gif_path, only_keyframes)

if __name__ == '__main__':
    args = parse_args()
    make_thumbnails(args.host, args.token, args.media, args.input, args.output, args.gif)



# coding: utf-8

"""
    Tator REST API

    Interface to the Tator backend.  # noqa: E501

    The version of the OpenAPI document: v1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from ..configuration import Configuration


class StateUpdate(object):
    """
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'attributes': 'dict(str, object)',
        'elemental_id': 'str',
        'frame': 'int',
        'in_place': 'int',
        'localization_ids_add': 'list[int]',
        'localization_ids_remove': 'list[int]',
        'mark': 'int',
        'null_attributes': 'list[str]',
        'parent': 'float',
        'pedantic': 'int',
        'reset_attributes': 'list[str]',
        'user_elemental_id': 'str'
    }

    attribute_map = {
        'attributes': 'attributes',
        'elemental_id': 'elemental_id',
        'frame': 'frame',
        'in_place': 'in_place',
        'localization_ids_add': 'localization_ids_add',
        'localization_ids_remove': 'localization_ids_remove',
        'mark': 'mark',
        'null_attributes': 'null_attributes',
        'parent': 'parent',
        'pedantic': 'pedantic',
        'reset_attributes': 'reset_attributes',
        'user_elemental_id': 'user_elemental_id'
    }

    def __init__(self, attributes=None, elemental_id=None, frame=None, in_place=0, localization_ids_add=None, localization_ids_remove=None, mark=None, null_attributes=None, parent=None, pedantic=0, reset_attributes=None, user_elemental_id=None, local_vars_configuration=None):  # noqa: E501
        """StateUpdate - a model defined in OpenAPI"""
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._attributes = None
        self._elemental_id = None
        self._frame = None
        self._in_place = None
        self._localization_ids_add = None
        self._localization_ids_remove = None
        self._mark = None
        self._null_attributes = None
        self._parent = None
        self._pedantic = None
        self._reset_attributes = None
        self._user_elemental_id = None
        self.discriminator = None

        if attributes is not None:
            self.attributes = attributes
        self.elemental_id = elemental_id
        if frame is not None:
            self.frame = frame
        if in_place is not None:
            self.in_place = in_place
        if localization_ids_add is not None:
            self.localization_ids_add = localization_ids_add
        if localization_ids_remove is not None:
            self.localization_ids_remove = localization_ids_remove
        if mark is not None:
            self.mark = mark
        if null_attributes is not None:
            self.null_attributes = null_attributes
        self.parent = parent
        if pedantic is not None:
            self.pedantic = pedantic
        if reset_attributes is not None:
            self.reset_attributes = reset_attributes
        if user_elemental_id is not None:
            self.user_elemental_id = user_elemental_id

    @property
    def attributes(self):
        """
        Object containing attribute values.

        :return: The attributes of this StateUpdate. 
        :rtype: dict(str, object)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """
        Object containing attribute values.

        :param attributes: The attributes of this StateUpdate.
        :type: dict(str, object)
        """

        self._attributes = attributes

    @property
    def elemental_id(self):
        """
        The elemental ID of the object.

        :return: The elemental_id of this StateUpdate. 
        :rtype: str
        """
        return self._elemental_id

    @elemental_id.setter
    def elemental_id(self, elemental_id):
        """
        The elemental ID of the object.

        :param elemental_id: The elemental_id of this StateUpdate.
        :type: str
        """

        self._elemental_id = elemental_id

    @property
    def frame(self):
        """
        Frame number this state applies to.

        :return: The frame of this StateUpdate. 
        :rtype: int
        """
        return self._frame

    @frame.setter
    def frame(self, frame):
        """
        Frame number this state applies to.

        :param frame: The frame of this StateUpdate.
        :type: int
        """

        self._frame = frame

    @property
    def in_place(self):
        """
        Set to 1 to edit this record in-place, else mark-based versioning rules apply. This changes to 1 for authorship changes automatically.

        :return: The in_place of this StateUpdate. 
        :rtype: int
        """
        return self._in_place

    @in_place.setter
    def in_place(self, in_place):
        """
        Set to 1 to edit this record in-place, else mark-based versioning rules apply. This changes to 1 for authorship changes automatically.

        :param in_place: The in_place of this StateUpdate.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                in_place is not None and in_place > 1):  # noqa: E501
            raise ValueError("Invalid value for `in_place`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                in_place is not None and in_place < 0):  # noqa: E501
            raise ValueError("Invalid value for `in_place`, must be a value greater than or equal to `0`")  # noqa: E501

        self._in_place = in_place

    @property
    def localization_ids_add(self):
        """
        List of new localization IDs that this state applies to.

        :return: The localization_ids_add of this StateUpdate. 
        :rtype: list[int]
        """
        return self._localization_ids_add

    @localization_ids_add.setter
    def localization_ids_add(self, localization_ids_add):
        """
        List of new localization IDs that this state applies to.

        :param localization_ids_add: The localization_ids_add of this StateUpdate.
        :type: list[int]
        """

        self._localization_ids_add = localization_ids_add

    @property
    def localization_ids_remove(self):
        """
        List of new localization IDs that this state applies to.

        :return: The localization_ids_remove of this StateUpdate. 
        :rtype: list[int]
        """
        return self._localization_ids_remove

    @localization_ids_remove.setter
    def localization_ids_remove(self, localization_ids_remove):
        """
        List of new localization IDs that this state applies to.

        :param localization_ids_remove: The localization_ids_remove of this StateUpdate.
        :type: list[int]
        """

        self._localization_ids_remove = localization_ids_remove

    @property
    def mark(self):
        """
        Revision number of this object on this version branch

        :return: The mark of this StateUpdate. 
        :rtype: int
        """
        return self._mark

    @mark.setter
    def mark(self, mark):
        """
        Revision number of this object on this version branch

        :param mark: The mark of this StateUpdate.
        :type: int
        """

        self._mark = mark

    @property
    def null_attributes(self):
        """
        Null a value in the attributes body

        :return: The null_attributes of this StateUpdate. 
        :rtype: list[str]
        """
        return self._null_attributes

    @null_attributes.setter
    def null_attributes(self, null_attributes):
        """
        Null a value in the attributes body

        :param null_attributes: The null_attributes of this StateUpdate.
        :type: list[str]
        """

        self._null_attributes = null_attributes

    @property
    def parent(self):
        """
        If a clone, the pk of the parent.

        :return: The parent of this StateUpdate. 
        :rtype: float
        """
        return self._parent

    @parent.setter
    def parent(self, parent):
        """
        If a clone, the pk of the parent.

        :param parent: The parent of this StateUpdate.
        :type: float
        """

        self._parent = parent

    @property
    def pedantic(self):
        """
        Set to 1 to enforce that this is the latest mark; else push edits to end of branch.

        :return: The pedantic of this StateUpdate. 
        :rtype: int
        """
        return self._pedantic

    @pedantic.setter
    def pedantic(self, pedantic):
        """
        Set to 1 to enforce that this is the latest mark; else push edits to end of branch.

        :param pedantic: The pedantic of this StateUpdate.
        :type: int
        """
        if (self.local_vars_configuration.client_side_validation and
                pedantic is not None and pedantic > 1):  # noqa: E501
            raise ValueError("Invalid value for `pedantic`, must be a value less than or equal to `1`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                pedantic is not None and pedantic < 0):  # noqa: E501
            raise ValueError("Invalid value for `pedantic`, must be a value greater than or equal to `0`")  # noqa: E501

        self._pedantic = pedantic

    @property
    def reset_attributes(self):
        """
        Reset an attribute to the default value specified in the Type object

        :return: The reset_attributes of this StateUpdate. 
        :rtype: list[str]
        """
        return self._reset_attributes

    @reset_attributes.setter
    def reset_attributes(self, reset_attributes):
        """
        Reset an attribute to the default value specified in the Type object

        :param reset_attributes: The reset_attributes of this StateUpdate.
        :type: list[str]
        """

        self._reset_attributes = reset_attributes

    @property
    def user_elemental_id(self):
        """
        Unique ID of the original user who created this. If permissions allow, will change the creating user to the one referenced by this elemental_id

        :return: The user_elemental_id of this StateUpdate. 
        :rtype: str
        """
        return self._user_elemental_id

    @user_elemental_id.setter
    def user_elemental_id(self, user_elemental_id):
        """
        Unique ID of the original user who created this. If permissions allow, will change the creating user to the one referenced by this elemental_id

        :param user_elemental_id: The user_elemental_id of this StateUpdate.
        :type: str
        """

        self._user_elemental_id = user_elemental_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, StateUpdate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, StateUpdate):
            return True

        return self.to_dict() != other.to_dict()
